@@ -55,7 +55,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                  data, enclos = sys.frame(sys.parent()))
   
   
-  if (!is.null(subset))
+  missing.subset <- is.null(subset)
+  if (!missing.subset)
     if ((is.logical(subset) & (sum(subset) > length(event.e))) ||
         (length(subset) > length(event.e)))
       stop("Length of subset is larger than number of studies.")
@@ -66,6 +67,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     byvar.name <- as.character(mf[[match("byvar", names(mf))]])
     if (length(byvar.name)>1 & byvar.name[1]=="$")
       byvar.name <- byvar.name[length(byvar.name)]
+    if (length(byvar.name)>1)
+      byvar.name <- "byvar"
   }
   
   
@@ -77,29 +80,45 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   
   if (keepdata){
     if (nulldata){
-      data <- data.frame(event.e=event.e, n.e=n.e,
-                         event.c=event.c, n.c=n.c,
-                         studlab=studlab)
+      data <- data.frame(.event.e=event.e, .n.e=n.e,
+                         .event.c=event.c, .n.c=n.c,
+                         .studlab=studlab)
       if (!missing.byvar)
-        data$byvar <- byvar
-      if (!is.null(subset))
-        data$subset <- subset
+        data$.byvar <- byvar
+      ##
+      if (!missing.subset){
+        if (length(subset) == dim(data)[1])
+          data$.subset <- subset
+        else{
+          data$.subset <- FALSE
+          data$.subset[subset] <- TRUE
+        }
+      }
     }
     else{
-      data$event.e <- event.e
-      data$n.e <- n.e
-      data$event.c <- event.c
-      data$n.c <- n.c
+      data$.event.e <- event.e
+      data$.n.e <- n.e
+      data$.event.c <- event.c
+      data$.n.c <- n.c
       ##
-      data$studlab <- studlab
+      data$.studlab <- studlab
       ##
       if (!missing.byvar)
-        data$byvar <- byvar
+        data$.byvar <- byvar
+      ##
+      if (!missing.subset){
+        if (length(subset) == dim(data)[1])
+          data$.subset <- subset
+        else{
+          data$.subset <- FALSE
+          data$.subset[subset] <- TRUE
+        }
+      }
     }
   }
   
   
-  if (!is.null(subset)){
+  if (!missing.subset){
     event.e <- event.e[subset]
     n.e <- n.e[subset]
     event.c <- event.c[subset]
@@ -661,6 +680,13 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   }
   
   
+  ##
+  ## Calculate H and I-Squared
+  ##
+  Hres  <- calcH(Q, df.Q, level.comb)
+  I2res <- isquared(Q, df.Q, level.comb)
+  
+  
   ##
   ## TODO - Is this check still necessary (version of meta >= 2.0) ???
   ## Modify fixed effects estimate:
@@ -712,6 +738,15 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
               k=k, Q=Q, df.Q=df.Q,
               tau=sqrt(tau2), se.tau2=se.tau2,
               C=Cval,
+              ##
+              H=Hres$TE,
+              lower.H=Hres$lower,
+              upper.H=Hres$upper,
+              ##
+              I2=I2res$TE,
+              lower.I2=I2res$lower,
+              upper.I2=I2res$upper,
+              ##
               Q.CMH=Q.CMH,
               sm=sm, method=method,
               sparse=sparse,
