@@ -18,19 +18,19 @@ print.metabias <- function(x, ...){
     if (length(x$k)!=0 & length(x$k.min!=0)){
       if (x$k <= x$k.min){
         if (x$k <= 2)
-          warning(paste("Number of studies (k=",  x$k,
-                        ") too small to test for small study effects.\n", sep=""))
+          warning("Number of studies (k=",  x$k,
+                  ") too small to test for small study effects.")
         else
-          warning(paste("Number of studies (k=",  x$k,
-                        ") too small to test for small study effects (k.min=", x$k.min, ").\n",
-                        "Change parameter 'k.min' if appropriate.\n", sep=""))
+          warning("Number of studies (k=",  x$k,
+                  ") too small to test for small study effects (k.min=",
+                  x$k.min, "). Change parameter 'k.min' if appropriate.\n")
       }
     }
     ##
     ## Check whether meta-analysis has subgroups:
     ##
     if (length(x$subgroup)!=0)
-      warning("No test for small study effects conducted for meta-analysis with subgroups.\n")  
+      warning("No test for small study effects conducted for meta-analysis with subgroups.")
   }
   
   invisible(NULL)
